import random

def GenerateSecret(length=16):
    letters ="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
    letters+="abcdefghijklmnopqrstuvwxyz"
    letters+="01234567890!@#$%^&*()"

    secret=""
    for i in range(length):
        secret+=random.choice(letters)

    return secret

    




from plone.openid.tests.oitestcase import OpenIdTestCase
from zExceptions import Redirect


class TestOpenIdExtraction(OpenIdTestCase):

    def testEmptyExtraction(self):
        """Test if we do not invent credentials out of thin air.
        """
        creds=self.app.folder.openid.extractCredentials(self.app.REQUEST)
        self.assertEqual(creds, {})


    def testRedirect(self):
        """Test if a redirect is generated for a login attempt.
        This test requires a working internet connection!
        """
        self.app.REQUEST.form["__ac_identity_url"]=self.identity
        self.assertRaises(Redirect,
                self.app.folder.openid.extractCredentials,
                self.app.REQUEST)


    def testPositiveOpenIdResponse(self):
        """Test if a positive authentication is extracted.
        """
        self.app.REQUEST.form.update(self.server_response)
        creds=self.app.folder.openid.extractCredentials(self.app.REQUEST)
        self.assertEqual(creds["openid.identity"], self.identity)
        self.assertEqual(creds["openid.mode"], "id_res")
        self.assertEqual(creds["openid.return_to"], "return_to")


    def testNegativeOpenIdResponse(self):
        """Check if a cancelled authentication request is correctly ignored.
        """
        self.app.REQUEST.form.update(self.server_response)
        self.app.REQUEST.form["openid.mode"]="cancel"
        creds=self.app.folder.openid.extractCredentials(self.app.REQUEST)
        self.assertEqual(creds, {})


    def testSessionCookie(self):
        """Check if a session cookie is found.
        """
        self.app.folder.openid.setupSession(self.identity)
        cookie=self.app.REQUEST["RESPONSE"].cookies[self.app.folder.openid.cookie_name]['value']
        self.app.REQUEST[self.app.folder.openid.cookie_name]=cookie

        creds=self.app.folder.openid.extractCredentials(self.app.REQUEST)
	identity=self.app.folder.openid.authenticateCredentials(creds)
        self.assertEqual(identity[0], self.identity)


    def testFormRedirectPriorities(self):
        """Check if a new login identity has preference over openid server
        reponse.
        """
        self.app.REQUEST.form.update(self.server_response)
        self.app.REQUEST.form["__ac_identity_url"]=self.identity
        self.assertRaises(Redirect,
                self.app.folder.openid.extractCredentials,
                self.app.REQUEST)

    def testFormCookiePriorities(self):
        """Check if a new login identity has preference over a session cookie.
        """
        self.app.folder.openid.setupSession(self.identity)
        cookie=self.app.REQUEST["RESPONSE"].cookies[self.app.folder.openid.cookie_name]['value']
        self.app.REQUEST[self.app.folder.openid.cookie_name]=cookie
        self.app.REQUEST.form["__ac_identity_url"]=self.identity
        self.assertRaises(Redirect,
                self.app.folder.openid.extractCredentials,
                self.app.REQUEST)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite=TestSuite()
    suite.addTest(makeSuite(TestOpenIdExtraction))
    return suite



from plone.openid.tests.oitestcase import OpenIdTestCase
from zExceptions import Redirect


class TestOpenIdAuthentication(OpenIdTestCase):

    def buildServerResponse(self):
        credentials={}
        for field in [ "nonce", "openid.assoc_handle", "openid.return_to",
                        "openid.signed", "openid.sig",
                        "openid.invalidate_handle", "openid.mode"]:
            credentials[field]=field
        credentials["openid.identity"]=self.identity
        credentials["openid.source"]="server"

        return credentials


    def testEmptyAuthentication(self):
        """Test if we do not invent an identity out of thin air.
        """
        creds=self.app.folder.openid.authenticateCredentials({})
        self.assertEqual(creds, None)


    def testUnknownOpenIdSource(self):
        """Test if an incorrect source does not produce unexpected exceptions.
        """
        creds=self.app.folder.openid.authenticateCredentials({"openid.source" : "x"})
        self.assertEqual(creds, None)


    def testServerAuthentication(self):
        """Test authentication of OpenID server responses.
        """
        credentials=self.buildServerResponse()
        creds=self.app.folder.openid.authenticateCredentials(credentials)
        self.assertEqual(creds, (self.identity, self.identity))


    def testIncompleteServerAuthentication(self):
        """Test authentication of OpenID server responses.
        """
        credentials=self.buildServerResponse()
        del credentials["openid.sig"]
        creds=self.app.folder.openid.authenticateCredentials(credentials)
        self.assertEqual(creds, None)


def test_suite():
    from unittest import TestSuite, makeSuite
    suite=TestSuite()
    suite.addTest(makeSuite(TestOpenIdAuthentication))
    return suite



from AccessControl.Permissions import manage_users as ManageUsers
from Products.PluggableAuthService.PluggableAuthService import registerMultiPlugin

from plone.openid.config import HAS_OPENID

if HAS_OPENID:
    from plugins import oid
    registerMultiPlugin(oid.OpenIdPlugin.meta_type)
else:
    import logging
    logger=logging.getLogger("Plone")
    logger.info("OpenID system packages not installed, OpenID support not available")


def initialize(context):
    if HAS_OPENID:
        context.registerClass(oid.OpenIdPlugin,
                                permission=ManageUsers,
                                constructors=
                                        (oid.manage_addOpenIdPlugin,
                                        oid.addOpenIdPlugin),
                                visibility=None,
                                icon="www/openid.png")




from plone.openid.util import GenerateSecret
from openid.store.interface import OpenIDStore
from openid.association import Association
from BTrees.OIBTree import OITreeSet
from BTrees.OOBTree import OOBTree
import time

class ZopeStore(OpenIDStore):
    """Zope OpenID store.

    This class implements an OpenID store which uses the ZODB.
    """
    def __init__(self):
        self.associations=OOBTree()
        self.handles=OOBTree()
        self.nonces=OITreeSet()
        self.authkey=GenerateSecret(length=self.AUTH_KEY_LEN)


    def getAssociationKey(self, server_url, handle):
        """Generate a key used to identify an association in our storage.
        """
        if handle is None:
            return self.handles[server_url][0]

        return (server_url, handle)


    def storeAssociation(self, server_url, association):
        key=self.getAssociationKey(server_url, association.handle)
        self.associations[key]=association.serialize()

        now=time.time()
        def cmpAssociations(one, two):
                return cmp(self.associations[one].getExpiresIn(now),
                           self.associations[two].getExpiresIn(now))
        lst=self.handles.get(server_url, [])
        lst.append(key)
        lst.sort(cmp=cmpAssociations)
        self.handles[server_url]=lst


    def getAssociation(self, server_url, handle=None):
        try:
            key=self.getAssociationKey(server_url, handle)
            assoc=Association.deserialize(self.associations[key])
        except KeyError:
            return None

        now=time.time()
        if assoc.getExpiresIn(now)==0:
            self.removeAssociation(server_url, handle)

        return assoc


    def removeAssociation(self, server_url, handle):
        key=self.getAssociationKey(server_url, handle)
        try:
            del self.associations[key]

            lst=self.handles[server_url]
            lst.remove(key)
            self.handles[server_url]=lst

            return True
        except KeyError:
            return False


    def storeNonce(self, nonce):
        self.nonces.insert(nonce)


    def useNonce(self, nonce):
        try:
            self.nonces.remove(nonce)
            return True
        except KeyError:
            return False


    def getAuthKey(self):
        return self.authkey


    def isDumb(self):
        return False

